@@ -21,7 +21,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::Canon;
 
-$VERSION = '1.54';
+$VERSION = '1.57';
 
 sub WriteCRW($$);
 sub ProcessCanonRaw($$$);
@@ -74,9 +74,7 @@ sub BuildMakerNotes($$$$$$);
     0x080a => {
         Name => 'CanonRawMakeModel',
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::CanonRaw::MakeModel',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::CanonRaw::MakeModel' },
     },
     0x080b => { Name => 'CanonFirmwareVersion', Writable => 'string[32]' },
     0x080c => { Name => 'ComponentVersion',     Writable => 'string'     }, #3
@@ -120,16 +118,12 @@ sub BuildMakerNotes($$$$$$);
     0x1029 => {
         Name => 'CanonFocalLength',
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Canon::FocalLength',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Canon::FocalLength' },
     },
     0x102a => {
         Name => 'CanonShotInfo',
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Canon::ShotInfo',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Canon::ShotInfo' },
     },
     0x102c => {
         Name => 'CanonColorInfo2',
@@ -142,9 +136,7 @@ sub BuildMakerNotes($$$$$$);
     0x102d => {
         Name => 'CanonCameraSettings',
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Canon::CameraSettings',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Canon::CameraSettings' },
     },
     0x1030 => { #4
         Name => 'WhiteSample',
@@ -157,9 +149,7 @@ sub BuildMakerNotes($$$$$$);
     0x1031 => {
         Name => 'SensorInfo',
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Canon::SensorInfo',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Canon::SensorInfo' },
     },
     # this tag has only be verified for the 10D in CRW files, but the D30 and D60
     # also produce CRW images and have CustomFunction information in their JPEG's
@@ -201,9 +191,7 @@ sub BuildMakerNotes($$$$$$);
     0x1038 => {
         Name => 'CanonAFInfo',
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Canon::AFInfo',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Canon::AFInfo' },
     },
     0x1093 => {
         Name => 'CanonFileInfo',
@@ -215,12 +203,7 @@ sub BuildMakerNotes($$$$$$);
     0x10a9 => {
         Name => 'ColorBalance',
         Writable => 0,
-        SubDirectory => {
-            # this offset is necessary because the table contains short rationals
-            # (4 bytes long) but the first entry is 2 bytes into the table.
-            Start => '2',
-            TagTable => 'Image::ExifTool::Canon::ColorBalance',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Canon::ColorBalance' },
     },
     0x10b5 => { #PH
         Name => 'RawJpgInfo',
@@ -245,9 +228,7 @@ sub BuildMakerNotes($$$$$$);
     0x1803 => { #3
         Name => 'ImageFormat',
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::CanonRaw::ImageFormat',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::CanonRaw::ImageFormat' },
     },
     0x1804 => { Name => 'RecordID', Writable => 'int32u' }, #3
     0x1806 => { #3
@@ -330,9 +311,7 @@ sub BuildMakerNotes($$$$$$);
         Name => 'ExposureInfo',
         Groups => { 1 => 'CIFF' }, # (only so CIFF shows up in group lists)
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::CanonRaw::ExposureInfo',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::CanonRaw::ExposureInfo' },
     },
     0x1834 => { #PH
         Name => 'CanonModelID',
@@ -348,9 +327,7 @@ sub BuildMakerNotes($$$$$$);
     0x1835 => {
         Name => 'DecoderTable',
         Writable => 0,
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::CanonRaw::DecoderTable',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::CanonRaw::DecoderTable' },
     },
     0x183b => { #PH
         # display format for serial number
@@ -634,13 +611,13 @@ sub AUTOLOAD
 # Returns: 1 on success
 sub ProcessCanonRaw($$$)
 {
-    my ($exifTool, $dirInfo, $rawTagTable) = @_;
+    my ($et, $dirInfo, $rawTagTable) = @_;
     my $blockStart = $$dirInfo{DirStart};
     my $blockSize = $$dirInfo{DirLen};
     my $raf = $$dirInfo{RAF} or return 0;
     my $buff;
-    my $verbose = $exifTool->Options('Verbose');
-    my $buildMakerNotes = $exifTool->Options('MakerNotes');
+    my $verbose = $et->Options('Verbose');
+    my $buildMakerNotes = $et->Options('MakerNotes');
 
     # 4 bytes at end of block give directory position within block
     $raf->Seek($blockStart+$blockSize-4, 0) or return 0;
@@ -652,7 +629,7 @@ sub ProcessCanonRaw($$$)
     # read the directory (10 bytes per entry)
     $raf->Read($buff, 10 * $entries) == 10 * $entries or return 0;
 
-    $verbose and $exifTool->VerboseDir('CIFF', $entries);
+    $verbose and $et->VerboseDir('CIFF', $entries);
     my $index;
     for ($index=0; $index<$entries; ++$index) {
         my $pt = 10 * $index;
@@ -661,13 +638,13 @@ sub ProcessCanonRaw($$$)
         my $valuePtr = Get32u(\$buff, $pt+6);
         my $ptr = $valuePtr + $blockStart;  # all pointers relative to block start
         if ($tag & 0x8000) {
-            $exifTool->Warn('Bad CRW directory entry');
+            $et->Warn('Bad CRW directory entry');
             return 1;
         }
         my $tagID = $tag & 0x3fff;          # get tag ID
         my $tagType = ($tag >> 8) & 0x38;   # get tag type
         my $valueInDir = ($tag & 0x4000);   # flag for value in directory
-        my $tagInfo = $exifTool->GetTagInfo($rawTagTable, $tagID);
+        my $tagInfo = $et->GetTagInfo($rawTagTable, $tagID);
         if (($tagType==0x28 or $tagType==0x30) and not $valueInDir) {
             # this type of tag specifies a raw subdirectory
             my $name;
@@ -684,13 +661,13 @@ sub ProcessCanonRaw($$$)
             );
             if ($verbose) {
                 my $fakeInfo = { Name => $name, SubDirectory => { } };
-                $exifTool->VerboseInfo($tagID, $fakeInfo,
+                $et->VerboseInfo($tagID, $fakeInfo,
                     'Index'  => $index,
                     'Size'   => $size,
                     'Start'  => $ptr,
                 );
             }
-            $exifTool->ProcessDirectory(\%subdirInfo, $rawTagTable);
+            $et->ProcessDirectory(\%subdirInfo, $rawTagTable);
             next;
         }
         my ($valueDataPos, $count, $subdir);
@@ -714,21 +691,21 @@ sub ProcessCanonRaw($$$)
             $valueDataPos = $ptr;
             if ($size <= 512 or ($verbose > 2 and $size <= 65536)
                 or ($tagInfo and ($$tagInfo{SubDirectory}
-                or grep(/^$$tagInfo{Name}$/i, $exifTool->GetRequestedTags()) )))
+                or grep(/^$$tagInfo{Name}$/i, $et->GetRequestedTags()) )))
             {
                 # read value if size is small or specifically requested
                 # or if this is a SubDirectory
                 unless ($raf->Seek($ptr, 0) and $raf->Read($value, $size) == $size) {
-                    $exifTool->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
+                    $et->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
                     next;
                 }
             } else {
                 $value = "Binary data $size bytes";
                 if ($tagInfo) {
-                    if ($exifTool->Options('Binary') or $verbose) {
+                    if ($et->Options('Binary') or $verbose) {
                         # read the value anyway
                         unless ($raf->Seek($ptr, 0) and $raf->Read($value, $size) == $size) {
-                            $exifTool->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
+                            $et->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
                             next;
                         }
                     }
@@ -750,7 +727,7 @@ sub ProcessCanonRaw($$$)
         if ($verbose) {
             my $val = $value;
             $format and $val = ReadValue(\$val, 0, $format, $count, $size);
-            $exifTool->VerboseInfo($tagID, $tagInfo,
+            $et->VerboseInfo($tagID, $tagInfo,
                 Table   => $rawTagTable,
                 Index   => $index,
                 Value   => $val,
@@ -763,7 +740,7 @@ sub ProcessCanonRaw($$$)
         }
         if ($buildMakerNotes) {
             # build maker notes information if requested
-            BuildMakerNotes($exifTool, $tagID, $tagInfo, \$value, $format, $count);
+            BuildMakerNotes($et, $tagID, $tagInfo, \$value, $format, $count);
         }
         next unless defined $tagInfo;
 
@@ -793,19 +770,20 @@ sub ProcessCanonRaw($$$)
                 DirLen   => $size - $subdirStart,
                 Nesting  => $$dirInfo{Nesting} + 1,
                 RAF      => $raf,
+                DirName  => $name,
                 Parent   => $$dirInfo{DirName},
             );
             #### eval Validate ($dirData, $subdirStart, $size)
             if (defined $$subdir{Validate} and not eval $$subdir{Validate}) {
-                $exifTool->Warn("Invalid $name data");
+                $et->Warn("Invalid $name data");
             } else {
-                $exifTool->ProcessDirectory(\%subdirInfo, $newTagTable, $$subdir{ProcessProc});
+                $et->ProcessDirectory(\%subdirInfo, $newTagTable, $$subdir{ProcessProc});
             }
         } else {
             # convert to specified format if necessary
             $format and $value = ReadValue(\$value, 0, $format, $count, $size);
             # save the information
-            $exifTool->FoundTag($tagInfo, $value);
+            $et->FoundTag($tagInfo, $value);
             delete $$tagInfo{RawConv} if $delRawConv;
         }
     }
@@ -818,10 +796,10 @@ sub ProcessCanonRaw($$$)
 # Returns: 1 if this was a valid Canon RAW file
 sub ProcessCRW($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my ($buff, $sig);
     my $raf = $$dirInfo{RAF};
-    my $buildMakerNotes = $exifTool->Options('MakerNotes');
+    my $buildMakerNotes = $et->Options('MakerNotes');
 
     $raf->Read($buff,2) == 2      or return 0;
     SetByteOrder($buff)           or return 0;
@@ -834,10 +812,10 @@ sub ProcessCRW($$)
     my $filesize = $raf->Tell()   or return 0;
 
     # initialize maker note data if building maker notes
-    $buildMakerNotes and InitMakerNotes($exifTool);
+    $buildMakerNotes and InitMakerNotes($et);
 
-    # set the FileType tag unless already done (ie. APP0 CIFF record in JPEG image)
-    $exifTool->SetFileType();
+    # set the FileType tag unless already done (eg. APP0 CIFF record in JPEG image)
+    $et->SetFileType();
 
     # build directory information for main raw directory
     my %dirInfo = (
@@ -851,17 +829,20 @@ sub ProcessCRW($$)
 
     # process the raw directory
     my $rawTagTable = GetTagTable('Image::ExifTool::CanonRaw::Main');
-    unless ($exifTool->ProcessDirectory(\%dirInfo, $rawTagTable)) {
-        $exifTool->Warn('CRW file format error');
+    my $oldIndent = $$et{INDENT};
+    $$et{INDENT} .= '| ';
+    unless (ProcessCanonRaw($et, \%dirInfo, $rawTagTable)) {
+        $et->Warn('CRW file format error');
     }
+    $$et{INDENT} = $oldIndent;
 
     # finish building maker notes if necessary
-    $buildMakerNotes and SaveMakerNotes($exifTool);
+    $buildMakerNotes and SaveMakerNotes($et);
 
     # process trailers if they exist in CRW file (not in CIFF information!)
-    if ($$exifTool{FILE_TYPE} eq 'CRW') {
+    if ($$et{FILE_TYPE} eq 'CRW') {
         my $trailInfo = Image::ExifTool::IdentifyTrailer($raf);
-        $exifTool->ProcessTrailers($trailInfo) if $trailInfo;
+        $et->ProcessTrailers($trailInfo) if $trailInfo;
     }
 
     return 1;
@@ -894,7 +875,7 @@ tags.)
 
 =head1 AUTHOR
 
-Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2015, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
