@@ -16,7 +16,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Sigma;
 
-$VERSION = '1.20';
+$VERSION = '1.23';
 
 sub ProcessX3FHeader($$$);
 sub ProcessX3FDirectory($$$);
@@ -33,6 +33,9 @@ sub ProcessX3FProperties($$$);
     Header => {
         SubDirectory => { TagTable => 'Image::ExifTool::SigmaRaw::Header' },
     },
+    Header4 => {
+        SubDirectory => { TagTable => 'Image::ExifTool::SigmaRaw::Header4' },
+    },
     HeaderExt => {
         SubDirectory => { TagTable => 'Image::ExifTool::SigmaRaw::HeaderExt' },
     },
@@ -61,6 +64,7 @@ sub ProcessX3FProperties($$$);
 %Image::ExifTool::SigmaRaw::Header = (
     PROCESS_PROC => \&ProcessX3FHeader,
     FORMAT => 'int32u',
+    NOTES => 'Information extracted from the header of an X3F file.',
     1 => {
         Name => 'FileVersion',
         ValueConv => '($val >> 16) . "." . ($val & 0xffff)',
@@ -89,9 +93,27 @@ sub ProcessX3FProperties($$$);
     },
 );
 
+# X3F version 4 header structure (ref PH)
+%Image::ExifTool::SigmaRaw::Header4 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    FORMAT => 'int32u',
+    NOTES => 'Header information for version 4.0 or greater X3F.',
+    1 => {
+        Name => 'FileVersion',
+        ValueConv => '($val >> 16) . "." . ($val & 0xffff)',
+    },
+    # 8 - undef[4]: 4 random ASCII characters
+    10 => 'ImageWidth',
+    11 => 'ImageHeight',
+    12 => 'Rotation',
+    # don't know what the rest of the header contains, but none of
+    # these values change in any of my samples...
+);
+
 # extended header tags
 %Image::ExifTool::SigmaRaw::HeaderExt = (
     GROUPS => { 2 => 'Camera' },
+    FORMAT => 'float',
     NOTES => 'Extended header data found in version 2.1 and 2.2 files',
     0 => 'Unused',
     1 => { Name => 'ExposureAdjust',PrintConv => 'sprintf("%.1f",$val)' },
@@ -242,108 +264,96 @@ sub ProcessX3FProperties($$$);
 
 #------------------------------------------------------------------------------
 # Extract null-terminated unicode string from list of characters
-# Inputs: 0) ExifTool object ref, 1) list ref, 2) position in list
+# Inputs: 0) ExifTool ref, 1) list ref, 2) position in list
 # Returns: Converted string
 sub ExtractUnicodeString($$$)
 {
-    my ($exifTool, $chars, $pos) = @_;
+    my ($et, $chars, $pos) = @_;
     my $i;
     for ($i=$pos; $i<@$chars; ++$i) {
         last unless $$chars[$i];
     }
     my $buff = pack('v*', @$chars[$pos..$i-1]);
-    return $exifTool->Decode($buff, 'UCS2', 'II');
+    return $et->Decode($buff, 'UCS2', 'II');
 }
 
 #------------------------------------------------------------------------------
 # Process an X3F header
-# Inputs: 0) ExifTool object reference, 1) DirInfo reference, 2) tag table ref
+# Inputs: 0) ExifTool ref, 1) DirInfo ref, 2) tag table ref
 # Returns: 1 on success
 sub ProcessX3FHeader($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $hdrLen = $$dirInfo{DirLen};
-    my $verbose = $exifTool->Options('Verbose');
 
     # process the static header structure first
-    $exifTool->ProcessBinaryData($dirInfo, $tagTablePtr);
+    $et->ProcessBinaryData($dirInfo, $tagTablePtr);
 
     # process extended data if available
-    if (length $$dataPt >= 232) {
+    if (length($$dataPt) - $hdrLen >= 160) {
+        my $verbose = $et->Options('Verbose');
         if ($verbose) {
-            $exifTool->VerboseDir('X3F HeaderExt', 32);
-            Image::ExifTool::HexDump($dataPt, undef,
+            $et->VerboseDir('X3F HeaderExt', 32);
+            HexDump($dataPt, undef,
                 MaxLen => $verbose > 3 ? 1024 : 96,
-                Out    => $exifTool->Options('TextOut'),
-                Prefix => $$exifTool{INDENT},
-                Start  => $$dirInfo{DirLen},
+                Out    => $et->Options('TextOut'),
+                Prefix => $$et{INDENT},
+                Start  => $hdrLen,
             ) if $verbose > 2;
         }
         $tagTablePtr = GetTagTable('Image::ExifTool::SigmaRaw::HeaderExt');
-        my @vals = unpack("x${hdrLen}C32V32", $$dataPt);
+        my @tags = unpack("x${hdrLen}C32", $$dataPt);
         my $i;
         my $unused = 0;
         for ($i=0; $i<32; ++$i) {
-            $vals[$i] or ++$unused, next;
-            my $val = $vals[$i+32];
-            # convert value 0x40000000 => 2 ** 1, 0x3f800000 => 2 ** 0, 0x3f000000 => 2 ** -1
-            if ($val) {
-                my $sign;
-                if ($val & 0x80000000) {
-                    $sign = -1;
-                    $val &= 0x7fffffff;
-                } else {
-                    $sign = 1;
-                }
-                $val = $sign * 2 ** (($val - 0x3f800000) / 0x800000);
-            }
-            $exifTool->HandleTag($tagTablePtr, $vals[$i], $val,
+            $tags[$i] or ++$unused, next;
+            $et->HandleTag($tagTablePtr, $tags[$i], undef,
                 Index  => $i,
                 DataPt => $dataPt,
                 Start  => $hdrLen + 32 + $i * 4,
                 Size   => 4,
             );
         }
-        $exifTool->VPrint(0, "$exifTool->{INDENT}($unused entries unused)\n");
+        $et->VPrint(0, "$$et{INDENT}($unused entries unused)\n");
     }
     return 1;
 }
 
 #------------------------------------------------------------------------------
 # Process an X3F properties
-# Inputs: 0) ExifTool object reference, 1) DirInfo reference, 2) tag table ref
+# Inputs: 0) ExifTool ref, 1) DirInfo ref, 2) tag table ref
 # Returns: 1 on success
 sub ProcessX3FProperties($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $size = length($$dataPt);
-    my $verbose = $exifTool->Options('Verbose');
-    my $unknown = $exifTool->Options('Unknown');
+    my $verbose = $et->Options('Verbose');
+    my $unknown = $et->Options('Unknown');
 
     unless ($size >= 24 and $$dataPt =~ /^SECp/) {
-        $exifTool->Warn('Bad properties header');
+        $et->Warn('Bad properties header');
         return 0;
     }
     my ($entries, $fmt, $len) = unpack('x8V2x4V', $$dataPt);
     unless ($size >= 24 + 8 * $entries + $len) {
-        $exifTool->Warn('Truncated Property directory');
+        $et->Warn('Truncated Property directory');
         return 0;
     }
-    $verbose and $exifTool->VerboseDir('Properties', $entries);
-    $fmt == 0 or $exifTool->Warn("Unsupported character format $fmt"), return 0;
+    $verbose and $et->VerboseDir('Properties', $entries);
+    $fmt == 0 or $et->Warn("Unsupported character format $fmt"), return 0;
     my $charPos = 24 + 8 * $entries;
     my @chars = unpack('v*',substr($$dataPt, $charPos, $len * 2));
     my $index;
     for ($index=0; $index<$entries; ++$index) {
         my ($namePos, $valPos) = unpack('V2',substr($$dataPt, $index*8 + 24, 8));
         if ($namePos >= @chars or $valPos >= @chars) {
-            $exifTool->Warn('Bad Property pointer');
+            $et->Warn('Bad Property pointer');
             return 0;
         }
-        my $tag = ExtractUnicodeString($exifTool, \@chars, $namePos);
-        my $val = ExtractUnicodeString($exifTool, \@chars, $valPos);
+        my $tag = ExtractUnicodeString($et, \@chars, $namePos);
+        my $val = ExtractUnicodeString($et, \@chars, $valPos);
         if (not $$tagTablePtr{$tag} and $unknown and $tag =~ /^\w+$/) {
             my $tagInfo = {
                 Name => "SigmaRaw_$tag",
@@ -355,7 +365,7 @@ sub ProcessX3FProperties($$$)
             AddTagToTable($tagTablePtr, $tag, $tagInfo);
         }
 
-        $exifTool->HandleTag($tagTablePtr, $tag, $val,
+        $et->HandleTag($tagTablePtr, $tag, $val,
             Index => $index,
             DataPt => $dataPt,
             Start => $charPos + 2 * $valPos,
@@ -367,12 +377,12 @@ sub ProcessX3FProperties($$$)
 
 #------------------------------------------------------------------------------
 # Write an X3F file
-# Inputs: 0) ExifTool object reference, 1) DirInfo reference (DirStart = directory offset)
+# Inputs: 0) ExifTool ref, 1) DirInfo ref (DirStart = directory offset)
 # Returns: error string, undef on success, or -1 on write error
 # Notes: Writes metadata to embedded JpgFromRaw image
 sub WriteX3F($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my $outfile = $$dirInfo{OutFile};
     my ($outDir, $buff, $ver, $entries, $dir, $outPos, $index, $didContain);
@@ -418,11 +428,11 @@ sub WriteX3F($$)
 
             # only rewrite full-sized JpgFromRaw (version 2.0, type 2, format 18)
             if ($buff =~ /^SECi\0\0\x02\0\x02\0\0\0\x12\0\0\0/ and
-                $$exifTool{ImageWidth} == unpack('x16V', $buff))
+                $$et{ImageWidth} == unpack('x16V', $buff))
             {
                 $raf->Read($buff, $len) == $len or return 'Error reading JpgFromRaw';
                 # use same write directories as JPEG
-                $exifTool->InitWriteDirs('JPEG');
+                $et->InitWriteDirs('JPEG');
                 # rewrite the embedded JPEG in memory
                 my $newData;
                 my %jpegInfo = (
@@ -430,14 +440,14 @@ sub WriteX3F($$)
                     RAF     => new File::RandomAccess(\$buff),
                     OutFile => \$newData,
                 );
-                $$exifTool{FILE_TYPE} = 'JPEG';
-                my $success = $exifTool->WriteJPEG(\%jpegInfo);
-                $$exifTool{FILE_TYPE} = 'X3F';
+                $$et{FILE_TYPE} = 'JPEG';
+                my $success = $et->WriteJPEG(\%jpegInfo);
+                $$et{FILE_TYPE} = 'X3F';
                 SetByteOrder('II');
                 return 'Error writing X3F JpgFromRaw' unless $success and $newData;
                 return -1 if $success < 0;
                 # write new data if anything changed, otherwise copy old image
-                my $outPt = $$exifTool{CHANGED} ? \$newData : \$buff;
+                my $outPt = $$et{CHANGED} ? \$newData : \$buff;
                 Write($outfile, $$outPt) or return -1;
                 # set $len to the total subsection data length
                 $len = length($$outPt) + 28;
@@ -462,7 +472,7 @@ sub WriteX3F($$)
         }
     }
     # warn if we couldn't add metadata to this image (should only be SD9 or SD10)
-    $didContain or $exifTool->Warn("Can't yet write SD9 or SD10 X3F images");
+    $didContain or $et->Warn("Can't yet write SD9 or SD10 X3F images");
     # write out the directory and the directory pointer, and we are done
     Write($outfile, $outDir, pack('V', $outPos)) or return -1;
     return undef;
@@ -470,13 +480,13 @@ sub WriteX3F($$)
 
 #------------------------------------------------------------------------------
 # Process an X3F directory
-# Inputs: 0) ExifTool object reference, 1) DirInfo reference, 2) tag table ref
+# Inputs: 0) ExifTool ref, 1) DirInfo ref, 2) tag table ref
 # Returns: error string or undef on success
 sub ProcessX3FDirectory($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $raf = $$dirInfo{RAF};
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
 
     $raf->Seek($$dirInfo{DirStart}, 0) or return 'Error seeking to directory start';
 
@@ -485,18 +495,18 @@ sub ProcessX3FDirectory($$$)
     $raf->Read($buff, 12) == 12 or return 'Truncated X3F image';
     $buff =~ /^SECd/ or return 'Bad section header';
     ($ver, $entries) = unpack('x4V2', $buff);
-    $verbose and $exifTool->VerboseDir('X3F Subsection', $entries);
+    $verbose and $et->VerboseDir('X3F Subsection', $entries);
     $raf->Read($dir, $entries * 12) == $entries * 12 or return 'Truncated X3F directory';
     for ($index=0; $index<$entries; ++$index) {
         my $pos = $index * 12;
         my ($offset, $len, $tag) = unpack("x${pos}V2a4", $dir);
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+        my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
         if ($verbose) {
-            $exifTool->VPrint(0, "$exifTool->{INDENT}$index) $tag Subsection ($len bytes):\n");
+            $et->VPrint(0, "$$et{INDENT}$index) $tag Subsection ($len bytes):\n");
             if ($verbose > 2) {
                 $raf->Seek($offset, 0) or return 'Error seeking';
                 $raf->Read($buff, $len) == $len or return 'Truncated image';
-                $exifTool->VerboseDump(\$buff);
+                $et->VerboseDump(\$buff);
             }
         }
         next unless $tagInfo;
@@ -507,10 +517,10 @@ sub ProcessX3FDirectory($$$)
             # ignore all image data but JPEG compressed (version 2.0, type 2, format 18)
             next unless $buff =~ /^SECi\0\0\x02\0\x02\0\0\0\x12\0\0\0/;
             # check preview image size and extract full-sized preview as JpgFromRaw
-            if ($$exifTool{ImageWidth} == unpack('x16V', $buff)) {
-                $$exifTool{IsJpgFromRaw} = 1;
-                $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
-                delete $$exifTool{IsJpgFromRaw};
+            if ($$et{ImageWidth} == unpack('x16V', $buff)) {
+                $$et{IsJpgFromRaw} = 1;
+                $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
+                delete $$et{IsJpgFromRaw};
             }
             $offset += 28;
             $len -= 28;
@@ -520,7 +530,7 @@ sub ProcessX3FDirectory($$$)
         if ($subdir) {
             my %dirInfo = ( DataPt => \$buff );
             my $subTable = GetTagTable($$subdir{TagTable});
-            $exifTool->ProcessDirectory(\%dirInfo, $subTable);
+            $et->ProcessDirectory(\%dirInfo, $subTable);
         } else {
             # extract metadata from JpgFromRaw
             if ($$tagInfo{Name} eq 'JpgFromRaw') {
@@ -528,12 +538,12 @@ sub ProcessX3FDirectory($$$)
                     Parent => 'X3F',
                     RAF    => new File::RandomAccess(\$buff),
                 );
-                $$exifTool{BASE} += $offset;
-                $exifTool->ProcessJPEG(\%dirInfo);
-                $$exifTool{BASE} -= $offset;
+                $$et{BASE} += $offset;
+                $et->ProcessJPEG(\%dirInfo);
+                $$et{BASE} -= $offset;
                 SetByteOrder('II');
             }
-            $exifTool->FoundTag($tagInfo, $buff);
+            $et->FoundTag($tagInfo, $buff);
         }
     }
     return undef;
@@ -541,54 +551,60 @@ sub ProcessX3FDirectory($$$)
 
 #------------------------------------------------------------------------------
 # Read/write information from a Sigma raw (X3F) image
-# Inputs: 0) ExifTool object reference, 1) DirInfo reference
+# Inputs: 0) ExifTool ref, 1) DirInfo ref
 # Returns: 1 on success, 0 if this wasn't a valid X3F image, or -1 on write error
 sub ProcessX3F($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $outfile = $$dirInfo{OutFile};
     my $raf = $$dirInfo{RAF};
     my $warn = $outfile ? \&Image::ExifTool::Error : \&Image::ExifTool::Warn;
-    my ($buff, $err);
+    my ($buff, $err, $hdrLen);
 
     return 0 unless $raf->Read($buff, 40) == 40;
     return 0 unless $buff =~ /^FOVb/;
 
     SetByteOrder('II');
-    $exifTool->SetFileType();
+    $et->SetFileType();
 
     # check version number
     my $ver = unpack('x4V',$buff);
     $ver = ($ver >> 16) . '.' . ($ver & 0xffff);
-    if ($ver > 3) {
-        &$warn($exifTool, "Untested X3F version ($ver). Please submit sample for testing", 1);
+    if ($ver > 5) {
+        &$warn($et, "Untested X3F version ($ver). Please submit sample for testing", 1);
     }
-    my $hdrLen = length $buff;
     # read version 2.1/2.2/2.3 extended header
     if ($ver > 2) {
-        $hdrLen += $ver > 2.2 ? 64 : 32;            # SceneCaptureType string added in 2.3
-        my $more = $hdrLen - length($buff) + 160;   # (extended header is 160 bytes)
-        my $buf2;
+        my ($extra, $buf2);
+        if ($ver >= 4) {
+            $hdrLen = 0x300;
+            $extra = 0;
+        } else {
+            $hdrLen = $ver > 2.2 ? 104 : 72;    # SceneCaptureType string added in 2.3
+            $extra = 160;                       # (extended header is 160 bytes)
+        }
+        my $more = $hdrLen - length($buff) + $extra;
         unless ($raf->Read($buf2, $more) == $more) {
-            &$warn($exifTool, 'Error reading extended header');
+            &$warn($et, 'Error reading X3F header');
             return 1;
         }
         $buff .= $buf2;
     }
+    my ($widPos, $hdrType) = $ver < 4 ? (28, 'Header') : (40, 'Header4');
     # extract ImageWidth for later
-    $$exifTool{ImageWidth} = Get32u(\$buff, 28);
+    $$et{ImageWidth} = Get32u(\$buff, $widPos);
     # process header information
     my $tagTablePtr = GetTagTable('Image::ExifTool::SigmaRaw::Main');
     unless ($outfile) {
-        $exifTool->HandleTag($tagTablePtr, 'Header', $buff,
+        $et->HandleTag($tagTablePtr, $hdrType, $buff,
             DataPt => \$buff,
             Size   => $hdrLen,
         );
     }
     # read the directory pointer
-    $raf->Seek(-4, 2) or &$warn($exifTool, 'Seek error'), return 1;
+    $raf->Seek(-4, 2) or &$warn($et, 'Seek error'), return 1;
     unless ($raf->Read($buff, 4) == 4) {
-        &$warn($exifTool, 'Error reading X3F dir pointer');
+        &$warn($et, 'Error reading X3F dir pointer');
         return 1;
     }
     my $offset = unpack('V', $buff);
@@ -598,13 +614,13 @@ sub ProcessX3F($$)
     );
     if ($outfile) {
         $dirInfo{OutFile} = $outfile;
-        $err = WriteX3F($exifTool, \%dirInfo);
+        $err = WriteX3F($et, \%dirInfo);
         return -1 if $err and $err eq '-1';
     } else {
         # process the X3F subsections
-        $err = $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+        $err = $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
     }
-    $err and &$warn($exifTool, $err);
+    $err and &$warn($et, $err);
     return 1;
 }
 
@@ -627,7 +643,7 @@ Sigma and Foveon X3F images.
 
 =head1 AUTHOR
 
-Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2015, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
